@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2008 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2012 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -20,7 +20,8 @@ use Socket;
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::defaults;
 
-our $VERSION = "0.91";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 our @ISA = qw(Exporter);
 
@@ -32,6 +33,8 @@ our @EXPORT = qw(
 		 rolllog_level
 		 rolllog_levels
 		 rolllog_num
+		 rolllog_gettz
+		 rolllog_settz
 		 rolllog_str
 			 LOG_NEVER
 			 LOG_TMI
@@ -92,6 +95,9 @@ my @logstrs =					# Valid strings for levels.
 
 my $logfile;					# rollerd's log file.
 
+my $DEFAULT_LOGTZ = 'gmt';			# Default timezone.
+my $usetz = $DEFAULT_LOGTZ;			# Timezone selector to use.
+
 
 
 ##############################################################################
@@ -208,6 +214,59 @@ sub rolllog_levels
 	return(@levels);
 }
 
+##############################################################################
+#
+# Routine:	rolllog_gettz()
+#
+# Purpose:	Return the timezone function used for timestamps in log
+#		messages.  'local' and 'gmt' are the acceptable values.
+#
+sub rolllog_gettz
+{
+	#
+	# If the logging timezone hasn't been set yet, we'll set it to
+	# the default value
+	#
+	if($usetz eq '')
+	{
+		$usetz = $DEFAULT_LOGTZ;
+	}
+
+	return($usetz);
+}
+
+##############################################################################
+#
+# Routine:	rolllog_settz()
+#
+# Purpose:	Set the timezone selector to use for timestamps in log
+#		messages.  'local' and 'gmt' are the acceptable values.
+#
+sub rolllog_settz
+{
+	my $newtz = shift;				# New timezone.
+	my $oldtz = $usetz;				# Old timezone.
+
+	#
+	# Ensure a valid timezone selector was given.  If no selector
+	# was given, then we'll use the default.
+	#
+	if($newtz ne '')
+	{
+		return('') if(($newtz !~ /^gmt$/i) && ($newtz !~ /^local$/i));
+	}
+	else
+	{
+		$newtz = dnssec_tools_default('log_tz');
+	}
+
+	#
+	# Set the timezone selector and return the old selector.
+	#
+	$usetz = lc $newtz;
+	return($oldtz);
+}
+
 ##############################################################################
 #
 # Routine:	rolllog_str()
@@ -410,7 +469,14 @@ sub rolllog_log
 	#
 	# Get the timestamp.
 	#
-	$kronos = gmtime();
+	if($usetz eq 'local')
+	{
+		$kronos = localtime();
+	}
+	else
+	{
+		$kronos = gmtime();
+	}
 	$kronos =~ s/^....//;
 
 	#
@@ -458,6 +524,9 @@ Net::DNS::SEC::Tools::rolllog - DNSSEC-Tools rollover logging interfaces.
   $bool = rolllog_validlevel($newlevel);
   $bool = rolllog_validlevel(8);
 
+  $curtz = rolllog_gettz();
+  $oldtz = rolllog_settz('local');
+
   rolllog_log(LOG_INFO,"example.com","zone is valid");
 
 =head1 DESCRIPTION
@@ -528,6 +597,23 @@ associated numeric log level.  The numeric log level is returned to the caller.
 
 If I<loglevel> is an invalid log level, -1 is returned.
 
+=item I<rolllog_gettz()>
+
+This routine returns the timezone selector currently in use.  This value may
+be either 'gmt' (for Greenwich Mean Time) or 'local' (for the host's local
+time.)
+
+=item I<rolllog_settz(tzsel)>
+
+This routine sets the timezone to be used for timestamps in messages written
+to the log.  This I<tzsel> value may be either 'gmt' (Greenwich Mean Time)
+or 'local' (for the host's local time.)  I<tzsel> may be uppercase or
+lowercase; the value will be converted to lowercase.  If no value is passed,
+then the default will be used.
+
+The current timezone selector is returned.  If an invalid selector is given,
+then an undefined value is returned.
+
 =item I<rolllog_str(loglevel)>
 
 This routine translates a log level (given in I<loglevel>) into the associated
@@ -568,7 +654,7 @@ numeric or textual value.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2008 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
