@@ -51,13 +51,29 @@
 
 #include <math.h>
 
+#ifdef __WIN32__
+#include <winsock2.h>
+#include <windns.h>
+#include <ws2tcpip.h>
+#define ns_t_a   DNS_TYPE_A
+#define ns_c_in  1
+#endif
+
+#ifndef __WIN32__
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
+#endif /* __WIN32__ */
 #include <validator/resolver.h>
 #include <validator/validator.h>
 #include <sys/types.h>
+#ifndef __WIN32__
 #include <sys/socket.h>
 #include <netdb.h>
+#endif /* __WIN32__ */
+
+#ifdef __WIN32__
+void WSAAPI freeaddrinfo (struct addrinfo*);
+#endif
 
 #include <QTimer>
 
@@ -75,7 +91,7 @@ GraphWidget::GraphWidget(QWidget *parent, QLineEdit *editor, const QString &file
     : QGraphicsView(parent), timerId(0), m_editor(editor),
       m_nodeScale(2), m_localScale(false), m_lockNodes(false), m_shownsec3(false),
       m_timer(0),
-      m_layoutType(springyLayout), m_childSize(30), m_lookupType(ns_t_a), m_animateNodeMovements(true),
+      m_layoutType(springyLayout), m_childSize(30), m_lookupType(1), m_animateNodeMovements(true),
       m_infoBox(infoBox), m_infoLabel(0), m_infoMoreButton(0), m_nodeInfoLabel(0), m_previousFileMenu(0), m_mapper(),
       m_nodeList(new NodeList(this)), m_logWatcher(new LogWatcher(this)), m_legend(0)
 {
@@ -388,8 +404,10 @@ void GraphWidget::doActualLookup(const QString &lookupString, int lookupType)
     busy();
 
     // perform the lookup
-    ret = val_res_query(NULL, lookupString.toUtf8(), ns_c_in,
+    qDebug() << "here: " << lookupString;
+    ret = val_res_query(NULL, lookupString.toUtf8(), 1,
                         lookupType, buf, sizeof(buf), &val_status);
+    qDebug() << "  result:" << ret;
 
     // do something with the results
     if (ret <= 0) {
@@ -427,7 +445,8 @@ void GraphWidget::doActualLookup(const QString &lookupString, int lookupType)
     if (node)
         m_nodeList->reApplyFiltersTo(node);
 
-    freeaddrinfo(aitop);
+    if (aitop)
+        freeaddrinfo(aitop);
     unbusy();
 }
 
@@ -634,7 +653,7 @@ void GraphWidget::about()
     msgBox.setWindowTitle("About DNSSEC-Nodes");
     msgBox.setText("<img src=\":/icons/dnssec-nodes-64x64.png\" style=\"float: right\"/>"
                    "<h1>DNSSEC-Nodes</h1><p><i>A component of the DNSSEC-Tools Project</i></p><p>For more information, please visit http://www.dnssec-tools.org/.</i></p>"
-                   "<p>DNSSEC-Tools Version: 1.11</p>");
+                   "<p>DNSSEC-Tools Version: 1.12</p>");
     msgBox.setStandardButtons(QMessageBox::Close);
     msgBox.exec();
 }
