@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 
@@ -7,7 +7,7 @@
  * Program to check if the provided dnsval.conf file is valid 
  *
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include <validator/validator.h>
 
 #ifdef HAVE_LIBGEN_H
@@ -37,7 +37,7 @@ void usage(char *progname)
     printf("        -h, --help                        Display this help and exit\n");
     printf("        -d, --dnsval-conf=<dnsval.conf>   Specify the dnsval.conf file\n");
     printf("        -r, --resolv-conf=<resolv.conf>   Specify the resolv.conf file\n");
-    printf("        -i, --root_h-nts=<root.hints>     Specify the root.hints file\n");
+    printf("        -i, --root-hints=<root.hints>     Specify the root.hints file\n");
     printf("        -v, --verbose                     Enable verbose mode\n");
 }
 
