@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 
@@ -13,7 +13,7 @@
  * It generates an output suitable for consumption by the
  * drawvalmap.pl script.  This output is written to stderr.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include <validator/validator.h>
 #include <validator/resolver.h>
 
@@ -73,86 +73,128 @@ sig_shutdown(int a)
     done = 1;
 }
 
+/*
+ * Returns:
+ *   0  expected results
+ *   1  untrusted / unexpected result
+ *  -1  bad parameter 
+ */
 int
 check_results(val_context_t * context, const char *desc, char * name,
               const u_int16_t class_h, const u_int16_t type_h,
               const int *result_ar, struct val_result_chain *results,
-              int trusted_only)
+              int trusted_only, struct timeval *start)
 {
     int             result_array[MAX_TEST_RESULTS];
-    int             err = 0, i;
+    int             extra_res[MAX_TEST_RESULTS];
+    int             err = 0, i, extra = 0, missing = 0, untrusted = 0;
+    struct timeval  now, duration;
     struct val_result_chain *res;
 
-    if ((NULL == result_ar) || (NULL == results))
+#define CR_EXPECTED  -1
+#define CR_UNTRUSTED -2
+#define CR_MISSING   -3
+
+    if (NULL == result_ar || NULL == start)
         return -1;
 
+    /** calculate query duration */
+    gettimeofday(&now, NULL);
+    timersub(&now, start, &duration);
+
     /*
      * make a local copy of result array 
      */
-    i = 0;
-    while (result_ar[i] != 0) {
+    for (i = 0; result_ar[i] != 0; ++i)
         result_array[i] = result_ar[i];
-        i++;
-    }
+
     result_array[i] = 0;
 
-    for (res = results; res && (err == 0); res = res->val_rc_next) {
-        for (i = 0; result_array[i] != 0; i++) {
-            if (res->val_rc_status == result_array[i]) {
-                result_array[i] = -1;   /* Mark this as done  */
+    /* 
+     * if we don't have any answers, the result type is VAL_UNTRUSTED_ANSWER */
+    if (results == NULL) {
+        for (i = 0; result_array[i] != 0; ++i) {
+            if (VAL_UNTRUSTED_ANSWER != result_array[i])
+                continue;
+            /* Mark this as done */
+            if (trusted_only) {
+                ++untrusted;
+                result_array[i] = CR_UNTRUSTED;
+            }
+            else
+                result_array[i] = CR_EXPECTED;
+        }
+    }
+
+    /** check for any untrusted results, if reqested */
+    if (trusted_only) {
+        for (res = results; res; res = res->val_rc_next) {
+            if (!val_istrusted(res->val_rc_status))
+                ++untrusted;
+        }
+    }
+    /** compare results we do have against what we expect */
+    fprintf(stderr, "%s: \t", desc);
+    if (result_array[0]) {
+        for (res = results; res; res = res->val_rc_next) {
+            for (i = 0; result_array[i] != 0; i++) {
+                if (res->val_rc_status != result_array[i])
+                    continue;
+                /* Mark this as done */
+                if (trusted_only && !val_istrusted(res->val_rc_status))
+                    result_array[i] = CR_UNTRUSTED;
+                else
+                    result_array[i] = CR_EXPECTED;
                 break;
             }
-        }
-        if (result_array[i] == 0) {
-            if (trusted_only) {
-                if (val_istrusted(res->val_rc_status)) {
-                    continue;
-                } else {
-                    err = 1;
-                }
-            } else {
-                fprintf(stderr, "%s: \t", desc);
-                fprintf(stderr,
-                        "FAILED: Remaining error values expected\n");
-                for (i = 0; result_array[i] != 0; i++) {
-                    if (result_array[i] != -1)
-                        fprintf(stderr, "     %s(%d)\n",
-                                p_val_error(result_array[i]),
-                                result_array[i]);
-                }
-                fprintf(stderr, "\n");
-                err = 1;
-            }
+            if (result_array[i] == 0) /* didn't expect this result */
+                extra_res[extra++] = res->val_rc_status;
         }
     }
 
     /*
-     * All results were in the result array 
+     * Check if any values were missing
      */
+    for (i = 0; result_array[i] != 0; ++i)
+        if (result_array[i] > 0)
+            ++missing;
+
+    /*
+     * print results
+     */
+    err = untrusted + extra + missing;
     if (!err) {
-        /*
-         * Check if all error values were marked 
-         */
-        for (i = 0; result_array[i] != 0; i++) {
-            if (result_array[i] != -1) {
-                fprintf(stderr, "%s: \t", desc);
-                fprintf(stderr,
-                        "FAILED: Some results were not received \n");
-                err = 1;
-                break;
-            }
+        fprintf(stderr, "OK\n");
+    } else {
+        fprintf(stderr, "FAILED: received results did not match expectations\n");
+
+        /** print extra */
+        if (extra) {
+            fprintf(stderr, "   Some results were not expected\n");
+            for (i = 0; i < extra; ++i)
+                fprintf(stderr,         "     UNEXPECTED %s(%d)\n",
+                        p_val_error(extra_res[i]), extra_res[i]);
         }
 
-        if (!err) {
-            fprintf(stderr, "%s: \t", desc);
-            fprintf(stderr, "OK\n");
-        }
-    } else if (trusted_only) {
-        fprintf(stderr, "%s: \t", desc);
-        fprintf(stderr,
-                "FAILED: Some results were not validated successfully \n");
+        /** print missing/untrusted */
+        if (trusted_only && untrusted)
+            fprintf(stderr, "   Some results were not validated successfully\n");
+        if (missing)
+            fprintf(stderr, "   Some results were not received\n");
+        if (missing || (trusted_only && untrusted))
+            for (i = 0; result_array[i] != 0; i++)
+                if (result_array[i] > 0)
+                    fprintf(stderr, "     MISSING    %s(%d)\n",
+                            p_val_error(result_array[i]), result_array[i]);
+                else if (result_array[i] == CR_UNTRUSTED)
+                    fprintf(stderr, "     UNTRUSTED  %s(%d)\n",
+                            p_val_error(result_ar[i]), result_ar[i]);
+
     }
 
+    fprintf(stderr, "%s: ****END**** %ld.%ld sec\n", desc, duration.tv_sec,
+            duration.tv_usec);
+
     return err;
 }
 
@@ -166,8 +208,10 @@ print_val_response(struct val_response *resp)
 
     printf("DNSSEC status: %s [%d]\n",
            p_val_error(resp->vr_val_status), resp->vr_val_status);
-    if (val_istrusted(resp->vr_val_status)) {
-        printf("Trusted response:\n");
+    if (val_isvalidated(resp->vr_val_status)) {
+        printf("Validated response:\n");
+    } else if (val_istrusted(resp->vr_val_status)) {
+        printf("Trusted but not validated response:\n");
     } else {
         printf("Untrusted response:\n");
     }
@@ -186,9 +230,9 @@ sendquery(val_context_t * context, const char *desc, char * name,
     int             ret_val;
     struct val_result_chain *results = NULL;
     int             err = 0;
-    struct timeval     now, start, duration;
+    struct timeval     start;
 
-    if ((NULL == desc) || (NULL == name) || (NULL == result_ar) )
+    if ((NULL == desc) || (NULL == name) || (NULL == result_ar) || (resp == NULL))
         return -1;
 
     fprintf(stderr, "%s: ****START**** \n", desc);
@@ -199,13 +243,12 @@ sendquery(val_context_t * context, const char *desc, char * name,
 
     if (ret_val == VAL_NO_ERROR) {
 
-        if (resp)
-            ret_val = compose_answer(name, type_h, class_h, results, resp);
+        ret_val = compose_answer(name, type_h, class_h, results, resp);
 
         if (result_ar)
             err =
                 check_results(context, desc, name, class_h, type_h,
-                              result_ar, results, trusted_only);
+                              result_ar, results, trusted_only, &start);
 
         val_free_result_chain(results);
     } else {
@@ -213,14 +256,8 @@ sendquery(val_context_t * context, const char *desc, char * name,
         fprintf(stderr, "FAILED: Error in val_resolve_and_check(): %s\n",
                 p_val_err(ret_val));
     }
-    gettimeofday(&now, NULL);
-
     results = NULL;
 
-    timersub(&now, &start, &duration);
-    fprintf(stderr, "%s: ****END**** %ld.%ld sec\n", desc, duration.tv_sec,
-            duration.tv_usec);
-
     return (err != 0);          /* 0 success, 1 error */
 }
 
@@ -245,6 +282,7 @@ usage(char *progname)
     printf("        -r, --resolv-conf=<file> Specifies a resolv.conf to search for nameservers\n");
     printf("        -i, --root-hints=<file> Specifies a root.hints to search for root nameservers\n");
     printf("        -I, --inflight=<number> Maximum number of simultaneous queries\n");
+    printf("        -m, --multi-thread=<number> Maximum number of simultaneous threads\n");
     printf("        -w, --wait=<secs> Run tests in a loop, sleeping for specifed seconds between runs\n");
     printf("        -l, --label=<label-string> Specifies the policy to use during validation\n");
     printf("        -o, --output=<debug-level>:<dest-type>[:<dest-options>]\n");
@@ -667,6 +705,7 @@ main(int argc, char *argv[])
     int             num_threads = 0;
     int             max_in_flight = 1;
     int             daemon = 0;
+    //u_int32_t       flags = VAL_QUERY_AC_DETAIL|VAL_QUERY_NO_EDNS0_FALLBACK;
     u_int32_t       flags = VAL_QUERY_AC_DETAIL;
     int             retvals[] = { 0 };
     int             tcs = 0, tce = -1;
@@ -799,11 +838,11 @@ main(int argc, char *argv[])
             break;
 
         case 'T':
-            tcs = strtol(optarg, &nextarg, 10) - 1;
+            tcs = strtol(optarg, &nextarg, 10);
             if (*nextarg == '\0')
                 tce = tcs;
             else
-                tce = atoi(++nextarg) - 1;
+                tce = atoi(++nextarg);
             break;
 
         case 'l':
@@ -838,9 +877,9 @@ main(int argc, char *argv[])
     context = NULL;
 #endif
 
-    /* returned level is 0 based;, > 6 means 8 or higher; e.g. -o 8:stdout */
+    /* returned level is 0 based;, > 6 means 7 or higher; e.g. -o 7:stdout */
     if (val_log_highest_debug_level() > 6)
-        res_io_set_debug(1);
+        res_set_debug_level(val_log_highest_debug_level());
 
     rc = 0;
 
@@ -853,6 +892,7 @@ main(int argc, char *argv[])
             goto done;
         } else {
 
+#ifndef VAL_NO_THREADS
             if (num_threads > 0) {
                 struct thread_params_st 
                     threadparams = {context, tcs, tce, flags, testcase_config,
@@ -861,13 +901,16 @@ main(int argc, char *argv[])
                 do_threads(num_threads, &threadparams);
                 fprintf(stderr, "Parent exiting\n");
             } else {
+#endif /* VAL_NO_THREADS */
                 do { /* endless loop */ 
                     rc = self_test(context, tcs, tce, flags, testcase_config,
                                    suite, doprint, max_in_flight);
                     if (wait)
                         sleep(wait);
                 } while (wait && !rc);
+#ifndef VAL_NO_THREADS
             }
+#endif /* VAL_NO_THREADS */
         }
 
         goto done;
@@ -875,6 +918,7 @@ main(int argc, char *argv[])
 
     domain_name = argv[optind++];
 
+#ifndef VAL_NO_THREADS
     if (num_threads > 0) {
         struct thread_params_st 
             threadparams = {context, tcs, tce, flags, testcase_config,
@@ -882,6 +926,7 @@ main(int argc, char *argv[])
 
         do_threads(num_threads, &threadparams);
     } else {
+#endif /* VAL_NO_THREADS */
         do { /* endless loop */
             one_test(context, domain_name, class_h, type_h, flags, retvals,
                      doprint);
@@ -890,7 +935,9 @@ main(int argc, char *argv[])
                 sleep(wait);
         } while (wait);
 
+#ifndef VAL_NO_THREADS
     }
+#endif /* VAL_NO_THREADS */
 
 done:
     if (context)
