@@ -51,7 +51,7 @@
  */
 
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef RESOLVER_H
@@ -133,10 +133,27 @@ struct name_server {
      */
 };
 
+/*
+ * ns_options values
+ */
+#define SR_QUERY_DEBUG              0x00000001
+#define SR_QUERY_RECURSE            0x00000002 
+#define SR_QUERY_SET_DO             0x00000004 
+#define SR_QUERY_SET_CD             0x00000008
+#define SR_QUERY_VALIDATING_STUB_FLAGS  (SR_QUERY_SET_DO | SR_QUERY_SET_CD) 
+#define SR_QUERY_DEFAULT                (SR_QUERY_RECURSE) 
+
+
 struct expected_arrival {
     SOCKET          ea_socket;
+#ifdef EA_EXTRA_DEBUG
+    char           *name;
+    u_int16_t       ea_type_h;
+    u_int16_t       ea_class_h;
+#endif
     struct name_server *ea_ns;
     int             ea_which_address;
+    int             ea_edns0_size;
     int             ea_using_stream;
     unsigned char   *ea_signed;
     size_t          ea_signed_length;
@@ -165,10 +182,14 @@ int             response_recv(int *trans_id,
                               struct timeval *closest_event,
                               struct name_server **respondent,
                               unsigned char ** answer, size_t * answer_length);
+int             res_response_checks(u_char ** answer, size_t * answer_length,
+                                    struct name_server **respondent);
 void            res_cancel(int *transaction_id);
-int             res_nsfallback(int transaction_id, struct timeval *closest_event,
+int             res_nsfallback(int transaction_id,
+                               struct timeval *closest_event,
+                               struct name_server *server,
                                const char *name, const unsigned short class_h,
-                               const unsigned short type_h, int *edns0);
+                               const unsigned short type_h);
 
 void            wait_for_res_data(fd_set * pending_desc,
                                   struct timeval *closest_event);
@@ -187,6 +208,7 @@ int             get_tcp(const char *name_n,
 void            print_response(unsigned char * ans, size_t resplen);
 
 struct sockaddr_storage **create_nsaddr_array(int num_addrs);
+struct name_server *create_name_server(void);
 struct name_server *parse_name_server(const char *cp,
                                       const char *name_n);
 int             clone_ns(struct name_server **cloned_ns,
@@ -196,10 +218,18 @@ int             clone_ns_list(struct name_server **ns_list,
 void            free_name_server(struct name_server **ns);
 void            free_name_servers(struct name_server **ns);
 
-void            res_io_set_debug(int val);
-int             res_io_get_debug(void);
+void            res_set_debug_level(int val);
+int             res_get_debug_level(void);
 void            res_io_view(void);
 
+int             label_bytes_cmp(const u_char * field1, size_t length1,
+                                const u_char * field2, size_t length2);
+int             labelcmp(const u_char * name1, const u_char * name2, 
+                        size_t label_cnt);
+int             namecmp(const u_char * name1, const u_char * name2);
+
+    int             res_map_srio_to_sr(int val);
+
 unsigned short       res_nametoclass(const char *buf, int *successp);
 unsigned short       res_nametotype(const char *buf, int *successp);
 
@@ -212,9 +242,9 @@ int res_io_check_one(struct expected_arrival *ea, struct timeval *next_evt,
  */
 
 int res_nsfallback_ea(struct expected_arrival *temp,
-                      struct timeval *closest_event,
+                      struct timeval *closest_event, struct name_server *server,
                       const char *name, const unsigned short class_h,
-                      const unsigned short type_h, int *edns0);
+                      const unsigned short type_h);
 
 struct expected_arrival *
 res_async_query_create(const char *name, const unsigned short type_h,
