@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VALIDATOR_H
@@ -47,8 +47,9 @@ struct timeval;
  */
 #define VAL_QFLAGS_ANY          0xffffffff
 /*
- * Flags in this bit mask are checked when finding
- * a matching record in the query cache 
+ * Flags in this bit mask MUST match when finding
+ * a record in the query cache. If they are not
+ * requested they cannot match. 
  */
 #define VAL_QFLAGS_CACHE_MASK       0x0000ffff
 #define VAL_QUERY_AC_DETAIL         0x00000001
@@ -57,29 +58,37 @@ struct timeval;
 #define VAL_QUERY_USING_DLV         0x00000008 
 #define VAL_QUERY_ASYNC             0x00000010
 #define VAL_QUERY_NO_EDNS0_FALLBACK 0x00000020
-#define VAL_QUERY_SKIP_CACHE        0x00000080
-#define VAL_QUERY_SKIP_RESOLVER     0x00000100
+#define VAL_QUERY_SKIP_RESOLVER     0x00000040
+#define VAL_QUERY_MARK_FOR_DELETION 0x00000080
+#define VAL_QUERY_IGNORE_SKEW       0x00000100
 
 /*
- * Flags in this bit mask are not checked when finding
+ * Flags in this bit mask are MUST match if they
+ * are requested, they MAY match if these flags are
+ * not requested 
+ */
+#define VAL_QFLAGS_CACHE_PREF_MASK  0x00ff0000
+#define VAL_QUERY_RECURSE           0x00010000
+#define VAL_QUERY_SKIP_CACHE        0x00020000
+
+/*
+ * Flags in this bit mask are ignored when finding
  * a matching record in the query cache 
  */
-#define VAL_QFLAGS_NOCACHE_MASK     0xffff0000
-#define VAL_QUERY_EDNS0_FALLBACK    0x00010000 
-#define VAL_QUERY_GLUE_REQUEST      0x00020000
-#define VAL_QUERY_REFRESH_QCACHE    0x00040000
-#define VAL_QUERY_RECURSE           0x00080000
-#define VAL_QUERY_NO_EDNS0          0x00100000
+#define VAL_QFLAGS_NOCACHE_MASK     0xff000000
+#define VAL_QUERY_EDNS0_FALLBACK    0x01000000 
+#define VAL_QUERY_GLUE_REQUEST      0x02000000
+
 
 #define VAL_QFLAGS_USERMASK (VAL_QUERY_AC_DETAIL |\
                              VAL_QUERY_DONT_VALIDATE |\
                              VAL_QUERY_NO_DLV |\
                              VAL_QUERY_ASYNC |\
                              VAL_QUERY_NO_EDNS0_FALLBACK |\
-                             VAL_QUERY_SKIP_CACHE |\
                              VAL_QUERY_SKIP_RESOLVER |\
                              VAL_QUERY_RECURSE |\
-                             VAL_QUERY_REFRESH_QCACHE)
+                             VAL_QUERY_SKIP_CACHE|\
+                             VAL_QUERY_IGNORE_SKEW)
 
 #ifndef LOG_EMERG
 #define LOG_EMERG 0
@@ -105,6 +114,14 @@ struct val_log;
 typedef struct val_log val_log_t;
 struct queries_for_query;
 
+/* validator context options */
+typedef struct val_context_opt {
+    unsigned int vc_flags;
+    char *vc_val_conf;
+    char *vc_res_conf;
+    char *vc_root_conf;
+} val_context_opt_t;
+
 /*
  * Validator policies can be one of the following
  */
@@ -301,12 +318,11 @@ struct queries_for_query;
     /*
      * asynchronous status
      */
-#define VAL_AS_CTX_USER_SUPPLIED     0x00000001 /* i.e. don't delete it! */
-#define VAL_AS_IGNORE_CACHE          0x00000002
-#define VAL_AS_NO_NEW_QUERIES        0x00000004
-#define VAL_AS_NO_ANSWERS            0x00000008 /* don't care about answers */
-#define VAL_AS_NO_CALLBACKS          0x00000010 /* don't call callbacks */
-#define VAL_AS_NO_CANCEL_CALLBACKS   0x00000020 /* no cb if req cancelled */
+#define VAL_AS_IGNORE_CACHE          0x00000001
+#define VAL_AS_NO_NEW_QUERIES        0x00000002
+#define VAL_AS_NO_ANSWERS            0x00000004 /* don't care about answers */
+#define VAL_AS_NO_CALLBACKS          0x00000008 /* don't call callbacks */
+#define VAL_AS_NO_CANCEL_CALLBACKS   0x00000010 /* no cb if req cancelled */
 
 #define VAL_AS_DONE                  0x01000000 /* have results/answers */
 #define VAL_AS_CALLBACK_CALLED       0x02000000 /* called user callbacks */
@@ -325,6 +341,7 @@ struct queries_for_query;
         char                    *name;
         int                      class_h;
         int                      type_h;
+        int                      retval;
         struct val_result_chain *results;
         struct val_answer_chain *answers;
     } val_cb_params_t;
@@ -343,7 +360,7 @@ struct queries_for_query;
                                          struct timeval *tv, unsigned int flags);
     int             val_async_select(val_context_t *context, fd_set *pending,
                                      int *nfds, struct timeval *timeout,
-                                     u_int32_t flags);
+                                     unsigned int flags);
     int             val_async_select_info(val_context_t *context,
                                     fd_set *fds,
                                     int *num_fds,
@@ -357,8 +374,8 @@ struct queries_for_query;
 
     int             val_async_cancel(val_context_t *context,
                                      val_async_status *as,
-                                     u_int flags);
-    int             val_async_cancel_all(val_context_t *context, u_int flags);
+                                     unsigned int flags);
+    int             val_async_cancel_all(val_context_t *context, unsigned int flags);
 
     /*
      * backwards compatibility
@@ -400,11 +417,20 @@ struct queries_for_query;
                                                  char *resolv_conf,
                                                  char *root_conf,
                                                  val_context_t ** newcontext);
+    int             val_create_context_ex(char *label,
+                                          val_context_opt_t *opt,
+                                          val_context_t ** newcontext);
     int             val_create_context(char *label,
                                        val_context_t ** newcontext);
     void            val_free_context(val_context_t * context);
     int             val_free_validator_state(void);
 
+#define VAL_CTX_FLAG_SET        0x01
+#define VAL_CTX_FLAG_RESET      0x02
+    int             val_context_setqflags(val_context_t *context,
+                                          unsigned char action,
+                                          unsigned int flags);
+
     /*
      * from val_policy.h 
      */
@@ -419,6 +445,7 @@ struct queries_for_query;
                                       val_policy_handle_t **pol);
     int             val_remove_valpolicy(val_context_t *context, 
                                       val_policy_handle_t *pol);
+    struct name_server *val_get_nameservers(val_context_t *ctx);
     /*
      * from val_x_query.c 
      */
@@ -499,7 +526,7 @@ struct queries_for_query;
                                            const struct addrinfo *hints_in,
                                            val_gai_callback callback,
                                            void *callback_data,
-                                           u_int vgai_flags,
+                                           unsigned int vgai_flags,
                                            val_gai_status **status);
 
     /*
@@ -546,7 +573,7 @@ struct queries_for_query;
                                    int class_h, int type_h,
                                    struct val_result_chain **results,
                                    struct val_answer_chain **answers,
-                                   u_int32_t vgafr_flags);
+                                   unsigned int vgafr_flags);
 
 
     /*
