@@ -14,7 +14,7 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "nsap_addr.h"
@@ -823,7 +823,7 @@ nxtbitmaps:
         int             n, m;
         char           *p;
 
-        len = SPRINTF((tmp, "\\# %u (\t; %s", edata - rdata, comment));
+        len = SPRINTF((tmp, "\\# %u (\t; %s", (unsigned)(edata - rdata), comment));
         T(addstr(tmp, len, &buf, &buflen));
         while (rdata < edata) {
             p = tmp;
