@@ -16,10 +16,10 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "res_mkquery.h"
@@ -393,52 +393,66 @@ int
 query_queue(const char *name, const u_int16_t type_h, const u_int16_t class_h,
             struct name_server *pref_ns, int *trans_id)
 {
+    struct expected_arrival *ea;
     int             ret_val;
 
-    u_char         *signed_query = NULL;
-    size_t          signed_length = 0;
-
-    struct name_server *ns_list = NULL;
-    struct name_server *ns;
-    long   delay = 0;
+    if (pref_ns == NULL || name == NULL || trans_id == NULL)
+        return SR_CALL_ERROR;
 
     *trans_id = -1;
 
-    if (pref_ns == NULL)
-        return SR_CALL_ERROR;
+    ea = res_async_query_create(name, type_h, class_h, pref_ns, 0);
+    if (NULL == ea)
+        return SR_MEMORY_ERROR;
 
-
-    /** res_io_stall(); */
-
-    /*
-     * clone these and store to ns_list 
-     */
-    if ((ret_val = clone_ns_list(&ns_list, pref_ns)) != SR_UNSET)
-        return ret_val;
-
-    /*
-     * Loop through the list of destinations 
-     */
-    for (ns = ns_list; ns; ns = ns->ns_next) {
-
-        /*
-         * Form the query with res_val_nmkquery_n 
-         */
-        if ((ret_val = res_create_query_payload(ns, name, class_h, type_h, 
-                        &signed_query, &signed_length)) < 0) {
-            continue;
-        }
-        if ((ret_val = res_io_queue(trans_id, signed_query,
-                                    signed_length, ns, delay)) < 0) {
-            continue;
-        }
-
-        delay += LIBSRES_NS_STAGGER;
-    }
+    ret_val = res_io_queue_ea(trans_id, ea);
+    if (ret_val != SR_IO_UNSET)
+        return SR_INTERNAL_ERROR;
 
     return SR_UNSET;
 }
 
+int
+res_response_checks(u_char ** answer, size_t * answer_length,
+                    struct name_server **respondent)
+{
+    int retval;
+
+    if (NULL == answer || NULL == answer_length)
+        return SR_INTERNAL_ERROR;
+
+    log_response(*answer, *answer_length);
+
+    if ((*respondent != NULL) && 
+        (res_tsig_verifies(*respondent, 
+                           *answer, *answer_length) != SR_TS_OK))
+        retval = SR_TSIG_ERROR;
+    else
+        retval = theres_something_wrong_with_header(*answer, *answer_length);
+
+    if (SR_UNSET == retval)
+        return retval;
+
+    res_log(NULL,LOG_DEBUG,"libsres: ""error in response; dropping; rc %d",
+            retval);
+    FREE(*answer);
+    *answer = NULL;
+    *answer_length = 0;
+
+    return SR_NO_ANSWER; 
+}
+
+/*
+ * Returns:
+ *    SR_INTERNAL_ERROR
+ *    SR_NO_ANSWER_YET
+ *    SR_NO_ANSWER
+ *    SR_GOT_ANSWER
+ *    SR_UNSET
+ *
+ * caller is responsible for releasing respondent, even when SR_NO_ANSWER
+ * is returned.
+ */
 int
 response_recv(int *trans_id,
               fd_set *pending_desc,
@@ -464,34 +478,9 @@ response_recv(int *trans_id,
     ret_val = res_io_accept(*trans_id, pending_desc, closest_event, 
                             answer, answer_length, respondent);
 
-    if (ret_val == SR_IO_NO_ANSWER_YET)
-        return SR_NO_ANSWER_YET;
+    ret_val = res_map_srio_to_sr(ret_val);
 
-    if (ret_val == SR_IO_NO_ANSWER)
-        return SR_NO_ANSWER;
-
-    if (ret_val == SR_IO_GOT_ANSWER) {
-        log_response(*answer, *answer_length);
-        if ((*respondent != NULL) && 
-            (res_tsig_verifies(*respondent, 
-                               *answer, *answer_length) == SR_TS_OK) && 
-            (theres_something_wrong_with_header(*answer, 
-                                                *answer_length) == SR_UNSET)) {
-            return SR_UNSET;
-
-        } else {
-            res_log(NULL,LOG_DEBUG,"libsres: ""error in response; dropping");
-            FREE(*answer);
-            *answer = NULL;
-            *answer_length = 0;
-            if (*respondent != NULL)
-                free_name_server(respondent);
-            *respondent = NULL;
-            return SR_NO_ANSWER; 
-        }
-    } 
-
-    return SR_NO_ANSWER_YET;
+    return ret_val;
 }
 
 int
@@ -507,11 +496,17 @@ get(const char *name,
     struct timeval closest_event;
     fd_set pending_desc;
     if (SR_UNSET == (ret_val = query_send(name, type_h, class_h, nslist, &trans_id))) {
+        if (server)
+            *server = NULL;
         res_log(NULL,LOG_DEBUG,"libsres: ""get %s", name);
         do {
             FD_ZERO(&pending_desc);
             timerclear(&closest_event);
 
+            if (server && NULL != *server) {
+                free_name_server(server);
+                *server = NULL;
+            }
             ret_val = response_recv(&trans_id, &pending_desc, &closest_event, server, response,
                                     response_length);
 
@@ -542,12 +537,18 @@ get_tcp(const char *name, u_int16_t type_h, u_int16_t class_h,
         return ret_val;
 
     res_switch_all_to_tcp_tid(trans_id);
+    if (server)
+        *server = NULL;
 
     res_log(NULL,LOG_DEBUG,"libsres: ""get_tcp %s", name);
     do {
         FD_ZERO(&pending_desc);
         timerclear(&closest_event);
 
+        if (server && NULL != *server) {
+            free_name_server(server);
+            *server = NULL;
+        }
         ret_val = response_recv(&trans_id, &pending_desc, &closest_event,
                                 server, response, response_length);
 
