@@ -16,15 +16,16 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include <openssl/rand.h>
 
 #include "res_support.h"
+#include "res_io_manager.h"
 
 extern void     libsres_pquery(const u_char * msg, size_t len, FILE * file);
 
@@ -134,12 +135,12 @@ print_hex_field(u_char field[], size_t length, size_t width, char *pref)
     start = 0;
     do {
         stop = (start + width) < length ? (start + width) : length;
-        printf(pref);
+        printf("%s", pref);
         for (i = start; i < stop; i++)
             printf("%02x ", (u_char) field[i]);
         printf("\n");
 
-        printf(pref);
+        printf("%s", pref);
         for (i = start; i < stop; i++)
             if (isprint(field[i]))
                 printf(" %c ", (u_char) field[i]);
@@ -194,6 +195,32 @@ create_nsaddr_array(int num_addrs)
     return ns_address;
 }
 
+struct name_server *
+create_name_server(void)
+{
+    struct name_server *ns;
+    ns = (struct name_server *) MALLOC(sizeof(struct name_server));
+    if (ns == NULL)
+        return NULL;
+
+    /*
+     * Initialize the rest of the fields
+     */
+    ns->ns_tsig = NULL;
+    ns->ns_security_options = ZONE_USE_NOTHING;
+    ns->ns_status = 0;
+
+    ns->ns_retrans = RES_TIMEOUT;
+    ns->ns_retry = RES_RETRY;
+    ns->ns_options = SR_QUERY_DEFAULT | SR_QUERY_DEBUG;
+    ns->ns_edns0_size = RES_EDNS0_DEFAULT;
+
+    ns->ns_next = NULL;
+    ns->ns_number_of_addresses = 0;
+
+    return ns;
+}
+
 struct name_server *
 parse_name_server(const char *cp, const char *name_n)
 {
@@ -217,7 +244,7 @@ parse_name_server(const char *cp, const char *name_n)
     if (cp ==  NULL)
         return NULL;
 
-    ns = (struct name_server *) MALLOC(sizeof(struct name_server));
+    ns = create_name_server();
     if (ns == NULL)
         return NULL;
 
@@ -229,21 +256,6 @@ parse_name_server(const char *cp, const char *name_n)
         return NULL;
     }
 
-    /*
-     * Initialize the rest of the fields
-     */
-    ns->ns_tsig = NULL;
-    ns->ns_security_options = ZONE_USE_NOTHING;
-    ns->ns_status = 0;
-
-    ns->ns_retrans = RES_TIMEOUT;
-    ns->ns_retry = RES_RETRY;
-    ns->ns_options = RES_DEFAULT | RES_DEBUG;
-    ns->ns_edns0_size = RES_EDNS0_DEFAULT;
-
-    ns->ns_next = NULL;
-    ns->ns_number_of_addresses = 0;
-
     /*
      * Look for port number in address string
      * syntax of '[address]:port'
@@ -341,14 +353,233 @@ u_int16_t libsres_random(void)
     return rnd;
 }
 
+int
+label_bytes_cmp(const u_char * field1, size_t length1, 
+                const u_char * field2, size_t length2)
+{
+    u_char        buffer1[NS_MAXCDNAME];
+    u_char        buffer2[NS_MAXCDNAME];
+    size_t        i;
+    size_t        min_len;
+    int           ret_val;
+
+    /*
+     * If the first n bytes are the same, then the length determines
+     * the difference - if any 
+     */
+    if (length1 == 0 || length2 == 0)
+        return length1 - length2;
+
+    min_len = (length1 < length2) ? length1 : length2;
+
+    /*
+     * Compare this label's first min_len bytes 
+     */
+    /*
+     * Convert to lower case first 
+     */
+    memcpy(buffer1, field1, min_len);
+    for (i = 0; i < min_len; i++)
+        if (isupper(buffer1[i]))
+            buffer1[i] = tolower(buffer1[i]);
+
+    memcpy(buffer2, field2, min_len);
+    for (i = 0; i < min_len; i++)
+        if (isupper(buffer2[i]))
+            buffer2[i] = tolower(buffer2[i]);
+
+    ret_val = memcmp(buffer1, buffer2, min_len);
+
+    /*
+     * If they differ, propgate that 
+     */
+    if (ret_val != 0)
+        return ret_val;
+    /*
+     * If the first n bytes are the same, then the length determines
+     * the difference - if any 
+     */
+    return length1 - length2;
+}
+
+int
+labelcmp(const u_char * name1, const u_char * name2, size_t label_cnt)
+{
+    /*
+     * Compare two names, assuming same number of labels in each 
+     */
+    size_t             length1;
+    size_t             length2;
+    const u_char  *ptr1[256];
+    const u_char  *ptr2[256];
+    size_t offset1 = 0;
+    size_t offset2 = 0;
+    size_t i;
+    
+    length1 = (int) (name1 ? name1[0] : 0);
+    length2 = (int) (name2 ? name2[0] : 0);
+
+    if (length1 == 0 || length2 == 0)
+        return length1 - length2;
+
+    if (label_cnt > 256) {
+        return -1;
+    }
+    
+    /* mark all the label start points */
+    for(i=0; i<label_cnt; i++) {
+        ptr1[i] = &name1[offset1];
+        ptr2[i] = &name2[offset2];
+        offset1 += name1[offset1]+1;
+        offset2 += name2[offset2]+1; 
+    }
+    
+    /* start from the last label, work upwards */
+    while (label_cnt > 0) {
+        int retval;
+
+        length1 = *ptr1[label_cnt-1];
+        length2 = *ptr2[label_cnt-1]; 
+
+        if (length1 == 0 || length2 == 0) {
+            retval = length1 - length2;
+        } else {
+            retval = label_bytes_cmp(&ptr1[label_cnt-1][1], 
+                                     length1,
+                                     &ptr2[label_cnt-1][1],
+                                     length2);
+        }
+
+        if (retval != 0)
+            return retval;
+
+        label_cnt--; 
+    }
+
+    /* all labels are identical */
+    return 0;
+}
+
+/*
+ * compare DNS wire format names
+ *
+ * returns
+ *      <0 if name1 is before name2
+ *       0 if equal
+ *      >0 if name1 is after name2
+ */
+int
+namecmp(const u_char * name1, const u_char * name2)
+{
+    size_t             labels1 = 1;
+    size_t             labels2 = 1;
+    size_t             index1 = 0;
+    size_t             index2 = 0;
+    size_t             i;
+    size_t             label_cnt;
+    size_t             ldiff;
+    int             ret_val;
+
+    /*
+     * deal w/any null ptrs 
+     */
+    if (name1 == NULL) {
+        if (name2 == NULL)
+            return 0;
+        else
+            return -1;
+    } else {
+        if (name2 == NULL)
+            return 1;
+    }
+
+    /*
+     * count labels 
+     */
+    for (; name1[index1]; index1 += (int) name1[index1] + 1)
+        labels1++;
+    for (; name2[index2]; index2 += (int) name2[index2] + 1)
+        labels2++;
+
+    index1 = 0;
+    index2 = 0;
+
+    /*
+     * find index in longer name where the number of labels is equal 
+     */
+    if (labels1 > labels2) {
+        label_cnt = labels2;
+        ldiff = labels1 - labels2;
+        for (i = 0; i < ldiff; i++)
+            index1 += name1[index1] + 1;
+    }
+    else {
+        label_cnt = labels1;
+        ldiff = labels2 - labels1;
+        for (i = 0; i < ldiff; i++)
+            index2 += name2[index2] + 1;
+    }
+
+    /*
+     * compare last N labels 
+     */
+    ret_val = labelcmp(&name1[index1], &name2[index2], label_cnt);
+
+    if (ret_val != 0)
+        return ret_val;
+
+    /*
+     * If one dname is a "proper suffix" of the other,
+     * the shorter comes first 
+     */
+    return labels1 - labels2;
+}
+
+int
+res_map_srio_to_sr(int val)
+{
+    switch(val) {
+        case  SR_IO_UNSET:
+        case  SR_IO_GOT_ANSWER:
+            val = SR_UNSET;
+            break;
+        case SR_IO_NO_ANSWER_YET:
+            val = SR_NO_ANSWER_YET;
+            break;
+        case  SR_IO_NO_ANSWER:
+            val = SR_NO_ANSWER;
+            break;
+        case SR_IO_MEMORY_ERROR:
+        case SR_IO_TOO_MANY_TRANS:
+        case SR_IO_SOCKET_ERROR:
+        case SR_IO_INTERNAL_ERROR:
+        default:
+            val = SR_INTERNAL_ERROR;
+    }
+    return val;
+}
+
 /*
  * using val_log for logging introduces a circular dependency. Default to
  * using stderr for logging unless USE_LIBVAL_LOGGING is defined.
  */
-#ifndef USE_LIBVAL_LOGGING
-
 static int sres_level = LOG_WARNING;
 
+void
+res_set_debug_level(int level)
+{
+    sres_level = level;
+}
+
+int
+res_get_debug_level(void)
+{
+    return sres_level;
+}
+
+
+#ifndef USE_LIBVAL_LOGGING
+
 void
 res_log(void *dont_care, int level, const char *template, ...)
 {
@@ -357,11 +588,7 @@ res_log(void *dont_care, int level, const char *template, ...)
     struct tm       *tm;
     va_list         ap;
 
-    if (NULL == template)
-        return;
-
-    /** check individual level */
-    if (level > sres_level)
+    if (NULL == template || level > sres_level)
         return;
 
     gettimeofday(&tv, NULL);
@@ -374,24 +601,56 @@ res_log(void *dont_care, int level, const char *template, ...)
     vsnprintf(&buf[19], sizeof(buf) - 21, template, ap);
     va_end(ap);
 
-    fprintf(stderr, buf);
-    fprintf(stderr, "\n");
+    fprintf(stderr, "%s\n", buf);
     fflush(stderr);
 }
 
+void
+res_log_ap(void *dont_care, int level, const char *template, va_list ap)
+{
+    char            buf[1028];
+    struct timeval  tv;
+    struct tm       *tm;
+
+    if (NULL == template || level > sres_level)
+        return;
+
+    gettimeofday(&tv, NULL);
+    tm = localtime(&tv.tv_sec);
+
+    snprintf(buf, sizeof(buf) - 2, "%04d%02d%02d::%02d:%02d:%02d ", 
+            tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
+            tm->tm_hour, tm->tm_min, tm->tm_sec);
+    vsnprintf(&buf[19], sizeof(buf) - 21, template, ap);
+
+    fprintf(stderr, "%s\n", buf);
+    fflush(stderr);
+}
+
+
 #else /* ifdef USE_LIBVAL_LOGGING */
 
+/** pass messages on to val_log... */
+void
+res_log_ap(void *dont_care, int level, const char *template, va_list ap)
+{
+    if (NULL == template || level > sres_level)
+        return;
+
+    val_log_ap((val_context_t*)dont_care, LOG_ERR, template, ap);
+}
+
 /** pass messages on to val_log... */
 void
 res_log(void *dont_care, int level, const char *template, ...)
 {
     va_list         ap;
 
-    if (NULL == template)
+    if (NULL == template || level > sres_level)
         return;
 
     va_start(ap, template);
-    val_log_ap((val_context_t*)dont_care, level, template, ap);
+    val_log_ap((val_context_t*)dont_care, LOG_ERR, template, ap);
     va_end(ap);
 }
 
