@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -12,7 +12,7 @@
  * DESCRIPTION
  * Contains implementation of val_getaddrinfo() and friends
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_policy.h"
@@ -539,7 +539,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
     int retval = EAI_FAIL;
 
     if (res == NULL)
-        return 0;
+        return EAI_FAIL;
 
     /* 
      * we may be calling get_addrinfo_from_result() repeatedly
@@ -1081,7 +1081,7 @@ val_getaddrinfo(val_context_t * context,
                                        hints, res, val_status);
     }
 
-    CTX_UNLOCK_POL(ctx); /* when was it locked? */
+    CTX_UNLOCK_POL(ctx);
     return retval;
 }                               /* val_getaddrinfo() */
 
@@ -1463,12 +1463,11 @@ int val_getaddrinfo_has_status(int rc) {
 #ifndef VAL_NO_ASYNC
 
 #define VAL_GAI_DONE         0x0001
-#define VAL_GAI_FREE_CONTEXT 0x0002
 
 struct val_gai_status_s {
     char             *nodename;
     char             *servname;
-    struct addrinfo  *hints;
+    const struct addrinfo  *hints;
 
     val_context_t    *context;
     struct addrinfo  *res;
@@ -1514,11 +1513,6 @@ _free_vgai( val_gai_status *vgai )
         vgai->res = NULL;
     }
 
-    if (vgai->flags & VAL_GAI_FREE_CONTEXT) {
-        val_free_context(vgai->context);
-        vgai->context = NULL;
-    }
-
     free(vgai);
 }
 
@@ -1530,16 +1524,18 @@ static int
 _vgai_async_callback(val_async_status *as, int event,
                      val_context_t *ctx, void *cb_data, val_cb_params_t *cbp)
 {
-    int                rc;
+    int                rc, gai_rc;
     val_gai_status    *vgai;
 
-    val_log(ctx, LOG_DEBUG, "val_getaddrinfo async callback for %p", as);
-
     vgai = (val_gai_status *)cb_data;
     if (NULL == vgai) {
         val_log(ctx, LOG_DEBUG, "val_getaddrinfo no callback data!");
         return VAL_NO_ERROR;
     }
+    val_log(ctx, LOG_DEBUG,
+            "val_getaddrinfo async callback for %p, %s %s(%d)", as,
+            vgai->nodename, p_type(cbp->type_h), cbp->type_h);
+
 
     if (0 == vgai->val_status) /* recently created */
         vgai->val_status = VAL_VALIDATED_ANSWER;
@@ -1558,14 +1554,12 @@ _vgai_async_callback(val_async_status *as, int event,
         /*
          * get addrinfo from results
          */
-        rc = get_addrinfo_from_result(ctx, cbp->answers, vgai->servname,
-                                      vgai->hints, &vgai->res,
-                                      &vgai->val_status);
-        if (VAL_NO_ERROR == rc) {
-            val_log(ctx, LOG_DEBUG,
-                    "val_gai_callback get_addrinfo_from_result() returned=%d with val_status=%d",
-                    rc, vgai->val_status);
-        }
+        gai_rc = get_addrinfo_from_result(ctx, cbp->answers, vgai->servname,
+                                          vgai->hints, &vgai->res,
+                                          &vgai->val_status);
+        val_log(ctx, LOG_DEBUG,
+                "val_gai_callback get_addrinfo_from_result() returned=%d with val_status=%d",
+                gai_rc, vgai->val_status);
     }
 
     /*
@@ -1642,7 +1636,7 @@ int
 val_getaddrinfo_submit(val_context_t * context, const char *nodename,
                        const char *servname, const struct addrinfo *hints_in,
                        val_gai_callback callback, void *callback_data,
-                       u_int val_gai_flags, val_gai_status **status)
+                       unsigned int val_gai_flags, val_gai_status **status)
 {
     val_gai_status        *vgai = NULL;
     int                    vretval = VAL_NO_ERROR, rc;
@@ -1658,8 +1652,6 @@ val_getaddrinfo_submit(val_context_t * context, const char *nodename,
     ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
     if (ctx == NULL)
         return VAL_INTERNAL_ERROR;
-    else if (ctx != context) /* we allocated our own context */
-        val_gai_flags |= VAL_GAI_FREE_CONTEXT;
 
     /*
      * use a default hints structure if one is not available.
@@ -1697,6 +1689,7 @@ val_getaddrinfo_submit(val_context_t * context, const char *nodename,
     vgai->flags = val_gai_flags;
     vgai->callback = callback;
     vgai->callback_data = callback_data;
+    vgai->hints = hints;
     if (servname)
         vgai->servname = (char *)strdup(servname);
     if (nodename)
