@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -11,7 +11,7 @@
  * This file contains the implementation for a validating gethostbyname function.
  * Applications should be able to use this with minimal change.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_policy.h"
@@ -721,6 +721,7 @@ val_gethostbyname2_r(val_context_t * context,
     }
     val_log(ctx, LOG_DEBUG, "val_gethostbyname2_r returned success, herrno = %d, val_status = %s", 
                 *h_errnop, val_status? p_val_status(*val_status) : NULL); 
+    CTX_UNLOCK_POL(ctx);
     return 0;
 
 err:
