@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -8,7 +8,7 @@
  * files and update the validator context.
  */
 
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_policy.h"
@@ -18,7 +18,11 @@
 #include "val_context.h"
 #include "val_assertion.h"
 #include "val_parse.h"
+#include "val_inline_conf.h"
 
+#ifdef ANDROID
+#include <sys/system_properties.h>
+#endif /* ANDROID */
 
 #define READ_POL_FOR_ZONE(buf_ptr, end_ptr, line_number, endst, retval, err, token)  do {\
     /*\
@@ -77,38 +81,38 @@
  * configuration and root.hints files.
  ***************************************************************
  */
-static char    *resolv_conf = NULL;
-static char    *root_hints = NULL;
-static char    *dnsval_conf = NULL;
+static char    *g_resolv_conf = NULL;
+static char    *g_root_hints = NULL;
+static char    *g_dnsval_conf = NULL;
 
 static int      atexit_reg = 0;
 
 static void
 policy_cleanup(void)
 {
-    if (NULL != resolv_conf)
-        free(resolv_conf);
+    if (NULL != g_resolv_conf)
+        free(g_resolv_conf);
 
-    if (NULL != root_hints)
-        free(root_hints);
+    if (NULL != g_root_hints)
+        free(g_root_hints);
 
-    if (NULL != dnsval_conf)
-        free(dnsval_conf);
+    if (NULL != g_dnsval_conf)
+        free(g_dnsval_conf);
 }
 
 
 char           *
 resolv_conf_get(void)
 {
-    if (NULL == resolv_conf) {
+    if (NULL == g_resolv_conf) {
         if (0 == atexit_reg) {
             atexit_reg = 1;
             atexit(policy_cleanup);
         }
-        resolv_conf = strdup(VAL_RESOLV_CONF);
+        g_resolv_conf = strdup(VAL_RESOLV_CONF);
     }
 
-    return strdup(resolv_conf);
+    return strdup(g_resolv_conf);
 }
 
 int
@@ -119,13 +123,13 @@ resolv_conf_set(const char *name)
     if (NULL == new_name)
         return 1;
 
-    if (NULL != resolv_conf)
-        free(resolv_conf);
+    if (NULL != g_resolv_conf)
+        free(g_resolv_conf);
     else if (0 == atexit_reg) {
         atexit_reg = 1;
         atexit(policy_cleanup);
     }
-    resolv_conf = new_name;
+    g_resolv_conf = new_name;
 
     return 0;
 }
@@ -133,16 +137,16 @@ resolv_conf_set(const char *name)
 char           *
 root_hints_get(void)
 {
-    if (NULL == root_hints) {
+    if (NULL == g_root_hints) {
         if (0 == atexit_reg) {
             atexit_reg = 1;
             atexit(policy_cleanup);
         }
 
-        root_hints = strdup(VAL_ROOT_HINTS);
+        g_root_hints = strdup(VAL_ROOT_HINTS);
     }
 
-    return strdup(root_hints);
+    return strdup(g_root_hints);
 }
 
 int
@@ -153,14 +157,14 @@ root_hints_set(const char *name)
     if (NULL == new_name)
         return 1;
 
-    if (NULL != root_hints)
-        free(root_hints);
+    if (NULL != g_root_hints)
+        free(g_root_hints);
     else if (0 == atexit_reg) {
         atexit_reg = 1;
         atexit(policy_cleanup);
     }
 
-    root_hints = new_name;
+    g_root_hints = new_name;
 
     return 0;
 }
@@ -168,10 +172,10 @@ root_hints_set(const char *name)
 char           *
 dnsval_conf_get(void)
 {
-    if (NULL == dnsval_conf)
-        dnsval_conf = strdup(VAL_CONFIGURATION_FILE);
+    if (NULL == g_dnsval_conf)
+        g_dnsval_conf = strdup(VAL_CONFIGURATION_FILE);
 
-    return strdup(dnsval_conf);
+    return strdup(g_dnsval_conf);
 }
 
 int
@@ -182,9 +186,9 @@ dnsval_conf_set(const char *name)
     if (NULL == new_name)
         return 1;
 
-    if (NULL != dnsval_conf)
-        free(dnsval_conf);
-    dnsval_conf = new_name;
+    if (NULL != g_dnsval_conf)
+        free(g_dnsval_conf);
+    g_dnsval_conf = new_name;
 
     return 0;
 }
@@ -624,6 +628,7 @@ val_get_token(char **buf_ptr,
     int             comment = 0;
     char            c;
     int             j = 0;
+    int            trail_space = -1;
 
     if ((buf_ptr == NULL) || (*buf_ptr == NULL) || 
         (end_ptr == NULL) || (line_number == NULL) ||
@@ -705,8 +710,12 @@ val_get_token(char **buf_ptr,
                         }
                     }     
                 }
-                if (!isspace(c))
+                if (!isspace(c)) {
                     escaped = 0;
+                    trail_space = -1;
+                } else {
+                    trail_space = i;
+                }
                 conf_token[i++] = c;
                 break;
         }
@@ -719,6 +728,11 @@ done:
 
         return VAL_CONF_PARSE_ERROR;
     }
+    /* remove all trailing white spaces */
+    if (trail_space != -1) {
+        conf_token[trail_space] = '\0';
+    }
+
     return VAL_NO_ERROR;
 }
 
@@ -1419,6 +1433,10 @@ destroy_valpol(val_context_t * ctx)
 #ifdef __OpenBSD__
 #define getprogname() NULL
 #endif
+#ifdef eabi
+#define getprogname() NULL
+#endif
+
 static int
 read_next_val_config_file(val_context_t *ctx, 
                           char **label, 
@@ -1436,6 +1454,7 @@ read_next_val_config_file(val_context_t *ctx,
     char token[TOKEN_MAX];
     int endst = 0;
     char *buf = NULL;
+    size_t bufsize = 0;
     char *buf_ptr, *end_ptr;
     int  line_number = 1;
     struct policy_fragment *pol_frag = NULL;
@@ -1460,44 +1479,56 @@ read_next_val_config_file(val_context_t *ctx,
     next_label = *label;
     
     fd = open(dnsval_c->dnsval_conf, O_RDONLY);
-    if (fd == -1) {
+    if (fd < 0) {
         val_log(ctx, LOG_ERR, 
                 "read_next_val_config_file(): Could not open validator conf file for reading: %s",
                 dnsval_c->dnsval_conf);
         /* check if we have at least one file */
-        if (dnsval_c == dlist) {
-            retval = VAL_CONF_NOT_FOUND;
-            goto err;
-        } else {
+        if (dnsval_c != dlist) {
             return VAL_NO_ERROR;
         }
-    }
+        /* check if we have the validator policy available inline */
+        bufsize = sizeof(val_conf_inline_buf);
+        if (!strncmp(val_conf_inline_buf, "", sizeof(""))) {
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        }
+        val_log(ctx, LOG_INFO, 
+                "read_next_val_config_file(): Using inline validator configuration data");
+        buf = (char *) MALLOC (bufsize * sizeof(char));
+        if (buf == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
+        memcpy(buf, val_conf_inline_buf, bufsize);
+    } else {
 #ifdef HAVE_FLOCK
-    memset(&fl, 0, sizeof(fl));
-    fl.l_type = F_RDLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        memset(&fl, 0, sizeof(fl));
+        fl.l_type = F_RDLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
-
-    if (0 != fstat(fd, &sb)) {
-        val_log(ctx, LOG_ERR, 
+        if (0 != fstat(fd, &sb)) {
+            val_log(ctx, LOG_ERR, 
                 "read_next_val_config_file(): Could not stat validator conf file: %s",
                 dnsval_c->dnsval_conf);
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
-    } 
-    dnsval_c->v_timestamp = sb.st_mtime;
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        } 
+        dnsval_c->v_timestamp = sb.st_mtime;
+        bufsize = sb.st_size;
 
-    buf = (char *) MALLOC (sb.st_size * sizeof(char));
-    if (buf == NULL) {
-        retval = VAL_OUT_OF_MEMORY;
-        goto err;
-    }
+        buf = (char *) MALLOC (bufsize * sizeof(char));
+        if (buf == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
 
-    if (-1 == read(fd, buf, sb.st_size)) {
-        val_log(ctx, LOG_ERR, "read_next_val_config_file(): Could not read validator conf file: %s",
-                dnsval_c->dnsval_conf);
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
+        if (-1 == read(fd, buf, bufsize)) {
+            val_log(ctx, LOG_ERR, "read_next_val_config_file(): Could not read validator conf file: %s",
+                    dnsval_c->dnsval_conf);
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        }
     }
 
     val_log(ctx, LOG_NOTICE, "read_next_val_config_file(): Reading validator policy from %s",
@@ -1520,7 +1551,7 @@ read_next_val_config_file(val_context_t *ctx,
         /* don't free global options g_opt since we're going to reuse this */
 
         buf_ptr = buf;
-        end_ptr = buf+sb.st_size;
+        end_ptr = buf+bufsize;
         dnsval_l = NULL;
         done = 1;
     
@@ -1549,6 +1580,9 @@ read_next_val_config_file(val_context_t *ctx,
                      * re-definition of global options 
                      * or global options was not in the first file
                      */
+                    val_log(ctx, LOG_WARNING, 
+                            "read_next_val_config_file(): Ignoring global options from line %d of %s",
+                            line_number, dnsval_c->dnsval_conf);
                     free_global_options(gt_opt);
                     FREE(gt_opt);
                     gt_opt = NULL;
@@ -1574,12 +1608,13 @@ read_next_val_config_file(val_context_t *ctx,
                     } 
                     if (gt_opt->app_policy == VAL_POL_GOPT_OVERRIDE ||
                             (*label == NULL && gt_opt->app_policy == VAL_POL_GOPT_ENABLE)) {
-                        next_label = getprogname(); 
-                        if (next_label != NULL) {
+                        const char *c_next_label = getprogname();
+                        if (c_next_label != NULL) {
                             val_log(ctx, LOG_NOTICE, 
                                     "read_next_val_config_file(): Using policy label from app name: %s",
-                                    next_label);
+                                    c_next_label);
                             done = 0;
+                            next_label = (char *)c_next_label;
                             break;
                         }
                         /* policy does not exist, dont create the impression that we have one */
@@ -1729,12 +1764,15 @@ read_next_val_config_file(val_context_t *ctx,
     *label = next_label;
     FREE(buf);
     buf = NULL;
+
+    if (fd > 0) {
 #ifdef HAVE_FLOCK
-    fl.l_type = F_UNLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
-    close(fd);
-    fd = -1;
+        close(fd);
+        fd = -1;
+    }
     return VAL_NO_ERROR;
 
 err:
@@ -1762,7 +1800,7 @@ err:
  * Precedence is environment, app and user
  */
 int
-read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
+read_val_config_file(val_context_t * ctx, char *scope)
 {
     struct policy_overrides *overrides;
     struct policy_overrides *t;
@@ -1775,7 +1813,7 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
     char *newctxlab;
     struct val_query_chain *q;
    
-    if (ctx == NULL || is_override == NULL)
+    if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
     
     if (ctx->dnsval_l && ctx->dnsval_l->dnsval_conf)
@@ -1797,7 +1835,6 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
 
     overrides = NULL;
     g_opt = NULL;
-    *is_override = 0;
 
     while(dnsval_c) {
         struct dnsval_list *added_list = NULL;
@@ -1850,19 +1887,10 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
         }
     }
 
-    if (label != scope) {
-        /* check if the pointer has changed */
-        *is_override = 1;
-    }
-
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_EX(ctx);
-
     if (ctx->label)
         FREE(ctx->label);
     ctx->label = newctxlab;
 
-
     destroy_valpol(ctx);
 
     /* Replace policies */
@@ -1898,17 +1926,16 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
     }
 
     /* 
-     * Re-initialize caches 
+     * Free the query cache 
      */
-    for(q = ctx->q_list; q; q=q->qc_next) {
-        q->qc_flags |= VAL_QUERY_REFRESH_QCACHE;
+    while (NULL != (q = ctx->q_list)) {
+        ctx->q_list = q->qc_next;
+        free_query_chain_structure(q);
+        q = NULL;
     }
 
     ctx->dnsval_l = dlist;
 
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_SH(ctx);
-
     val_log(ctx, LOG_DEBUG, "read_val_config_file(): Done reading validator configuration");
 
     return VAL_NO_ERROR;
@@ -1936,6 +1963,86 @@ destroy_respol(val_context_t * ctx)
     }
 }
 
+struct name_server *
+val_get_nameservers(val_context_t *context)
+{
+    val_context_t *ctx = NULL;
+    struct name_server *ns_list = NULL;
+
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL) {
+        return NULL;
+    }
+    clone_ns_list(&ns_list, ctx->nslist);
+
+    CTX_UNLOCK_POL(ctx);
+
+    return ns_list;
+}
+
+#ifdef ANDROID
+int
+read_res_config_file(val_context_t * ctx)
+{
+    /* android stores its resolvers in its property system */
+    char property_buffer[PROP_VALUE_MAX + 1];
+    char property_name[PROP_NAME_MAX+1];
+    int  property_buffer_len;
+    int  counter = 0;
+    struct name_server *ns_head = NULL;
+    struct name_server *ns_tail = NULL;
+    struct name_server *ns = NULL;
+
+    while(counter < 255) { /* arbitary way-too-high upper limit */
+        /* shouldn't be necessary, but still wise */
+        property_name[PROP_NAME_MAX] = '\0';
+        property_buffer[PROP_VALUE_MAX] = '\0';
+
+        snprintf(property_name, PROP_NAME_MAX, "net.dns%d", ++counter);
+        if (! __system_property_get(property_name, property_buffer)) {
+            /* end of the list: 0 length return */
+            break;
+        }
+
+        ns = parse_name_server(property_buffer, NULL);
+        if (ns == NULL) {
+            val_log(ctx, LOG_WARNING,
+                    "read_res_config_file(): error parsing android resource!");
+            return VAL_CONF_PARSE_ERROR;
+        }
+
+        ns->ns_options |= RES_RECURSE;
+        if (ns_tail == NULL) {
+            ns_head = ns;
+            ns_tail = ns;
+        } else {
+            ns_tail->ns_next = ns;
+            ns_tail = ns;
+        }
+    }
+
+    /*
+     * Check if we have root hints 
+     */
+    if (ns_head == NULL) {
+        if (!ctx->root_ns) {
+            val_log(ctx, LOG_WARNING, 
+                    "read_res_config_file(): Resolver configuration empty or missing, but root-hints was not found");
+            return VAL_CONF_NOT_FOUND;
+        }
+    } 
+
+    destroy_respol(ctx);
+    ctx->nslist = ns_head;
+    ctx->r_timestamp = 0; /* XXX: set to what?  there is no file stat */
+
+    val_log(ctx, LOG_DEBUG, 
+            "read_res_config_file(): Done reading resolver configuration");
+    return VAL_NO_ERROR;
+}
+
+#else /* ! ANDROID */
+
 int
 read_res_config_file(val_context_t * ctx)
 {
@@ -1954,68 +2061,84 @@ read_res_config_file(val_context_t * ctx)
     struct stat sb;
     char *buf_ptr, *end_ptr;
     char *buf = NULL;
+    size_t bufsize = 0;
     int retval;
-
+    time_t mtime = 0;
+    
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
 
-    resolv_config = ctx->resolv_conf;
-    if (NULL == resolv_config) {
-        if (!ctx->root_ns) {
-            val_log(ctx, LOG_WARNING, 
-                    "read_res_config_file(): Resolver configuration is NULL and root-hints was not found");
-            return VAL_CONF_NOT_FOUND;
-        }
-        val_log(ctx, LOG_DEBUG, "read_res_config_file(): No resolv.conf file configured, but root-hints available"); 
-        return VAL_NO_ERROR;
-    }
+    fd = -1;
 
-    fd = open(resolv_config, O_RDONLY);
-    if (fd == -1) {
-        val_log(ctx, LOG_ERR, "read_res_config_file(): Could not open resolver conf file for reading: %s",
-                resolv_conf);
-    
-        /* Use default resolv.conf file */
-        FREE(ctx->resolv_conf);
-        ctx->resolv_conf = strdup(VAL_DEFAULT_RESOLV_CONF);
-        if (ctx->resolv_conf == NULL) {
-            return VAL_OUT_OF_MEMORY;
-        }
-        resolv_config = ctx->resolv_conf;
+    resolv_config = ctx->resolv_conf;
+    if (resolv_config) {
         fd = open(resolv_config, O_RDONLY);
         if (fd == -1) {
-            val_log(ctx, LOG_ERR, "read_res_config_file(): Could not open default resolver conf file for reading: %s",
-                    resolv_conf);
-            return VAL_CONF_NOT_FOUND;
+            val_log(ctx, LOG_ERR, "read_res_config_file(): Could not open resolver conf file for reading: %s",
+                resolv_config);
+    
+            /* Use default resolv.conf file */
+            FREE(ctx->resolv_conf);
+            ctx->resolv_conf = strdup(VAL_DEFAULT_RESOLV_CONF);
+            if (ctx->resolv_conf == NULL) {
+                return VAL_OUT_OF_MEMORY;
+            }
+            resolv_config = ctx->resolv_conf;
+            fd = open(resolv_config, O_RDONLY);
         }
     }
+
+    if (fd > 0) {
 #ifdef HAVE_FLOCK
-    fl.l_type = F_RDLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        fl.l_type = F_RDLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
 
-    if (0 != fstat(fd, &sb)) {
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
-    } 
-    if (0 == sb.st_size)
-        goto empty;
+        if (0 != fstat(fd, &sb)) {
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        } 
+        if (0 == sb.st_size)
+            goto empty;
 
-    buf = (char *) MALLOC (sb.st_size * sizeof(char));
-    if (buf == NULL) {
-        retval = VAL_OUT_OF_MEMORY;
-        goto err;
+        mtime = sb.st_mtime;
+        bufsize = sb.st_size;
+        buf = (char *) MALLOC (bufsize * sizeof(char));
+        if (buf == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
+        if (-1 == read(fd, buf, bufsize)) {
+            val_log(ctx, LOG_ERR, "read_res_config_file(): Could not read resolver conf file: %s",
+                    resolv_config);
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        }
+        val_log(ctx, LOG_NOTICE, "read_res_config_file(): Reading resolver policy from %s", resolv_config);
+
+    } else {
+        if (resolv_config)
+            val_log(ctx, LOG_ERR, "read_res_config_file(): Could not open resolver conf file for reading: %s",
+                    resolv_config);
+
+        /* Try to read any inline root.hints information */
+        mtime = 0;
+        bufsize = sizeof(resolv_conf_inline_buf);
+        if (!strncmp(resolv_conf_inline_buf, "", sizeof(""))) {
+            goto empty;
+        }
+        val_log(ctx, LOG_INFO, 
+                "read_res_config_file(): Using inline resolv.conf data");
+        buf = (char *) MALLOC (bufsize * sizeof(char));
+        if (buf == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
+        memcpy(buf, resolv_conf_inline_buf, bufsize);
     }
+
     buf_ptr = buf;
-    end_ptr = buf+sb.st_size;
-
-    if (-1 == read(fd, buf, sb.st_size)) {
-        val_log(ctx, LOG_ERR, "read_res_config_file(): Could not read resolver conf file: %s",
-                resolv_conf);
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
-    }
-    val_log(ctx, LOG_NOTICE, "read_res_config_file(): Reading resolver policy from %s", resolv_config);
+    end_ptr = buf+bufsize;
 
     while(buf_ptr < end_ptr) {
 
@@ -2052,7 +2175,7 @@ read_res_config_file(val_context_t * ctx)
                 goto err;
 	        }
             if (ns != NULL) {
-                ns->ns_options |= RES_RECURSE;
+                ns->ns_options |= SR_QUERY_RECURSE;
                 if (ns_tail == NULL) {
                     ns_head = ns;
                     ns_tail = ns;
@@ -2109,12 +2232,15 @@ read_res_config_file(val_context_t * ctx)
     }
 
     FREE(buf);
+
   empty:
+    if (fd > 0 ) {
 #ifdef HAVE_FLOCK
-    fl.l_type = F_UNLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
-    close(fd);
+        close(fd);
+    }
 
     /*
      * Check if we have root hints 
@@ -2122,18 +2248,14 @@ read_res_config_file(val_context_t * ctx)
     if (ns_head == NULL) {
         if (!ctx->root_ns) {
             val_log(ctx, LOG_WARNING, 
-                    "read_res_config_file(): Resolver configuration is empty, but root-hints was not found");
+                    "read_res_config_file(): Resolver configuration empty or missing, but root-hints was not found");
             return VAL_CONF_NOT_FOUND;
         }
     } 
 
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_EX(ctx);
     destroy_respol(ctx);
     ctx->nslist = ns_head;
-    ctx->r_timestamp = sb.st_mtime;
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_SH(ctx);
+    ctx->r_timestamp = mtime;
 
     val_log(ctx, LOG_DEBUG, 
             "read_res_config_file(): Done reading resolver configuration");
@@ -2158,6 +2280,8 @@ read_res_config_file(val_context_t * ctx)
     return VAL_CONF_PARSE_ERROR;
 }
 
+#endif /* ! ANDROID */
+
 /*
  * parse the contents of the root.hints file into resource records 
  */
@@ -2188,6 +2312,8 @@ read_root_hints_file(val_context_t * ctx)
     int have_type;
     char *buf_ptr, *end_ptr;
     char *buf = NULL;
+    size_t bufsize = 0;
+    time_t mtime;
 
     class_h = 0;
     have_type = 0;
@@ -2196,48 +2322,63 @@ read_root_hints_file(val_context_t * ctx)
         return VAL_BAD_ARGUMENT;
     
     root_hints = ctx->root_conf;
-    /* 
-     *  Root hints are not necessary. Only needed if our resolv.conf is empty. 
-     * Flag the error at that time
-     */
-    if (NULL == root_hints) {
-        val_log(ctx, LOG_INFO, "read_root_hints_file(): No root.hints file configured"); 
-        return VAL_NO_ERROR;
-    }
 
-    fd = open(root_hints, O_RDONLY);
-    if (fd == -1) {
-        val_log(ctx, LOG_INFO, "read_root_hints_file(): Could not open root hints file for reading: %s",
-                root_hints);
-        return VAL_NO_ERROR;
-    }
+    fd = -1;
+
+    if (NULL != root_hints && 
+            (fd = open(root_hints, O_RDONLY)) > 0) {
 #ifdef HAVE_FLOCK
-    fl.l_type = F_RDLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        fl.l_type = F_RDLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
-
-    if (0 != fstat(fd, &sb)) { 
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
-    }
-
-    buf = (char *) MALLOC (sb.st_size * sizeof(char));
-    if (buf == NULL) {
-        retval = VAL_OUT_OF_MEMORY;
-        goto err;
-    }
-    buf_ptr = buf;
-    end_ptr = buf+sb.st_size;
-
-    if (-1 == read(fd, buf, sb.st_size)) {
-        val_log(ctx, LOG_ERR, "read_root_hints_file(): Could not read root hints file: %s",
+        if (0 != fstat(fd, &sb)) { 
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        }
+        mtime = sb.st_mtime;
+        bufsize = sb.st_size;
+        buf = (char *) MALLOC (bufsize * sizeof(char));
+        if (buf == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
+        if (-1 == read(fd, buf, bufsize)) {
+            val_log(ctx, LOG_ERR, "read_root_hints_file(): Could not read root hints file: %s",
+                    root_hints);
+            retval = VAL_CONF_NOT_FOUND;
+            goto err;
+        }
+        val_log(ctx, LOG_NOTICE, "read_root_hints_file(): Reading root hints from %s",
                 root_hints);
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
+
+    } else {
+        /* Try to read any inline root.hints information */
+        mtime = 0;
+        bufsize = sizeof(root_hints_inline_buf);
+        if (!strncmp(root_hints_inline_buf, "", sizeof(""))) {
+            if (root_hints)
+                val_log(ctx, LOG_INFO, "read_root_hints_file(): Could not open root hints file for reading: %s",
+                    root_hints);
+            else
+                val_log(ctx, LOG_INFO, "read_root_hints_file(): No root.hints file configured"); 
+            /* 
+             * Root hints are not necessary. Only needed if our resolv.conf is empty. 
+             * Flag the error at that time
+             */
+            return VAL_NO_ERROR;
+        }
+        val_log(ctx, LOG_INFO, 
+                "read_root_hints_file(): Using inline root.hints data");
+        buf = (char *) MALLOC (bufsize * sizeof(char));
+        if (buf == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
+        memcpy(buf, root_hints_inline_buf, bufsize);
     }
 
-    val_log(ctx, LOG_NOTICE, "read_root_hints_file(): Reading root hints from %s",
-            root_hints);
+    buf_ptr = buf;
+    end_ptr = buf+bufsize;
 
     while (buf_ptr < end_ptr) {
 
@@ -2339,12 +2480,26 @@ read_root_hints_file(val_context_t * ctx)
         if (type_h == ns_t_a) {
             struct sockaddr_in sa;
             size_t addrlen4 = sizeof(struct sockaddr_in);
-            if (INET_PTON(AF_INET, token, ((struct sockaddr *)&sa), &addrlen4) != 1) {
+            if ((addrlen4 == addrlen4) && /* this is to remove unused variable warning */
+                (INET_PTON(AF_INET, token, ((struct sockaddr *)&sa), &addrlen4) != 1)) {
                 retval = VAL_CONF_PARSE_ERROR;
                 goto err;
             }
             rdata_len_h = sizeof(struct in_addr);
             memcpy(rdata_n, &sa.sin_addr, rdata_len_h);
+#ifdef VAL_IPV6
+        } else if (type_h == ns_t_aaaa) {
+            struct sockaddr_in6 sa6;
+            size_t addrlen6 = sizeof(struct sockaddr_in6);
+            if ((addrlen6 == addrlen6) && /* this is to remove unused variable warning */
+                (INET_PTON(AF_INET6, token, ((struct sockaddr *)&sa6), &addrlen6) != 1)) {
+                val_log(ctx, LOG_INFO, 
+                        "read_root_hints_file(): Cannot parse IPv6 address, skipping.");
+                continue;
+            }
+            rdata_len_h = sizeof(struct in6_addr);
+            memcpy(rdata_n, &sa6.sin6_addr, rdata_len_h);
+#endif
         } else if (type_h == ns_t_ns) {
             if (ns_name_pton(token, rdata_n, sizeof(rdata_n)) == -1) {
                 retval = VAL_CONF_PARSE_ERROR;
@@ -2404,24 +2559,23 @@ read_root_hints_file(val_context_t * ctx)
     }
 #endif
 
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_EX(ctx);
     if (ctx->root_ns)
         free_name_servers(&ctx->root_ns);
     ctx->root_ns = ns_list;
-    ctx->h_timestamp = sb.st_mtime;
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_SH(ctx);
+    ctx->h_timestamp = mtime;
 
     res_sq_free_rrset_recs(&root_info);
 
     val_log(ctx, LOG_DEBUG, "read_root_hints_file(): Done reading root hints");
     FREE(buf);
+
+    if (fd > 0) {
 #ifdef HAVE_FLOCK
-    fl.l_type = F_UNLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
-    close(fd);
+        close(fd);
+    }
 
     return VAL_NO_ERROR;
 
@@ -2429,14 +2583,16 @@ read_root_hints_file(val_context_t * ctx)
 
     if (buf)
         FREE(buf);
+    if (fd > 0) {
 #ifdef HAVE_FLOCK
-    fl.l_type = F_UNLCK;
-    fcntl(fd, F_SETLKW, &fl);
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLKW, &fl);
 #endif
-    close(fd);
+        close(fd);
+    }
     res_sq_free_rrset_recs(&root_info);
-    val_log(ctx, LOG_ERR, "read_root_hints_file(): Error encountered while reading file %s - %s", 
-            root_hints, p_val_err(retval));
+    val_log(ctx, LOG_ERR, "read_root_hints_file(): Error encountered around line %d while reading file %s - %s",
+            line_number, root_hints, p_val_err(retval));
     return retval;
 }
 
@@ -2676,24 +2832,22 @@ val_add_valpolicy(val_context_t *context,
     }
    
     /* Lock exclusively */
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_EX(ctx);
+    CTX_LOCK_ACACHE(ctx);
 
-    /* Flush queries that match this name */
-    for(q=ctx->q_list; q; q=q->qc_next) {
-        /* Should never fail when holding above locks */
-        if (NULL != namename(q->qc_name_n, zone_n)) {
-            q->qc_flags |= VAL_QUERY_REFRESH_QCACHE;
-            if (pol_entry->exp_ttl > 0)
-                q->qc_ttl_x = pol_entry->exp_ttl;
-        }
-    }
     (*pol)->pe = pol_entry;
     (*pol)->index = index;
 
     /* Merge this policy into the context */
     STORE_POLICY_ENTRY_IN_LIST(pol_entry, ctx->e_pol[index]);
+
+    /* Flush queries that match this name */
+    for(q=ctx->q_list; q; q=q->qc_next) {
+        if (NULL != namename(q->qc_name_n, zone_n)) {
+            q->qc_flags |= VAL_QUERY_MARK_FOR_DELETION;
+        }
+    }
     
+    CTX_UNLOCK_ACACHE(ctx);
     CTX_UNLOCK_POL(ctx);
 
     return VAL_NO_ERROR;
@@ -2715,8 +2869,7 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
         return VAL_INTERNAL_ERROR;
     
     /* Lock exclusively */
-    CTX_UNLOCK_POL(ctx);
-    CTX_LOCK_POL_EX(ctx);
+    CTX_LOCK_ACACHE(ctx);
 
     /* find this policy in the context */
     prev = NULL;
@@ -2739,22 +2892,22 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
     }
     p->next = NULL;
 
-    /* Flush queries that match this name */
-    for(q=ctx->q_list; q; q=q->qc_next) {
-        /* Should never fail when holding above locks */
-        if (NULL != namename(q->qc_name_n, p->zone_n)) {
-            q->qc_flags |= VAL_QUERY_REFRESH_QCACHE;
-        }
-    }
-    
     /* free the policy */
     conf_elem_array[pol->index].free(p);
     FREE(p);
     FREE(pol);
     
+    /* Flush queries that match this name */
+    for(q=ctx->q_list; q; q=q->qc_next) {
+        if (NULL != namename(q->qc_name_n, p->zone_n)) {
+            q->qc_flags |= VAL_QUERY_MARK_FOR_DELETION;
+        }
+    }
+    
     retval = VAL_NO_ERROR;
 
 err:
+    CTX_UNLOCK_ACACHE(ctx);
     CTX_UNLOCK_POL(ctx);
     
     return retval;
