@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VAL_RESQUERY_H
@@ -12,8 +12,8 @@
                                    type_h != ns_t_ds) ? 1 : 0)
 
 struct glue_fetch_bucket {
-    struct queries_for_query *qfq[MAX_GLUE_FETCH_DEPTH];
-    int qfq_count;
+    struct queries_for_query *qfq;
+    struct glue_fetch_bucket *next_dep;
     struct glue_fetch_bucket *next_bucket;
 };
 
@@ -52,6 +52,7 @@ int             val_resquery_rcv(val_context_t * context,
 void            val_res_cancel(struct val_query_chain *matched_q);
 void            val_res_nsfallback(val_context_t *context, 
                                    struct val_query_chain *matched_q,
+                                   struct name_server *server,
                                    const char *name_p,
                                    struct timeval *closest_event);
 
